@@ -550,12 +550,14 @@
 "image.kriging" <-
   function (x, locations, borders, 
             values = x$predict, coords.data,
-            xlim, ylim, 
+            xlim, ylim,
             x.leg, y.leg, cex.leg = 0.8, vertical = FALSE, ...) 
 {
   pty.prev <- par()$pty
   ldots <- list(...)
   if(missing(x)) x <- NULL
+  attach(x)
+  on.exit(detach(x))
   if(missing(locations)) locations <-  eval(attr(x, "prediction.locations"))
   if(is.null(locations)) stop("prediction locations must be provided")
   if(ncol(locations) != 2)
@@ -574,7 +576,8 @@
                                      xlim = xlim, ylim = ylim) 
   par(pty = "s")
   image(x=locations$x, y=locations$y, z=locations$values,
-        xlim = locations$coords.lims[,1], ylim = locations$coords.lims[,2], ...)
+        xlim = locations$coords.lims[,1],
+        ylim = locations$coords.lims[,2], ...)
   ##
   ## adding points at data locations
   ##
@@ -590,8 +593,7 @@
     if(is.null(ldots$col)) ldots$col <- heat.colors(12)
     legend.krige(x.leg=x.leg, y.leg=y.leg,
                  values=locations$values[!is.na(locations$values)],
-                 vertical = vertical, cex=cex.leg,
-                 col=ldots$col)
+                 vertical = vertical, cex=cex.leg, ...)
   }
   par(pty = pty.prev)
   return(invisible())
@@ -601,6 +603,8 @@
   function(x, locations, borders, values = x$predict, ...)
 {
   if(missing(x)) x <- NULL
+  attach(x)
+  on.exit(detach(x))
   if(missing(locations)) locations <-  eval(attr(x, "prediction.locations"))
   if(is.null(locations)) stop("prediction locations must be provided")
   if(ncol(locations) != 2)
@@ -618,55 +622,50 @@
   values <- values[!is.na(values)]
   if(length(x.leg) != 2 | length(y.leg) != 2)
     stop("x.leg and y.leg require a vector with 2 elements")
-  v.r <- range(values, na.rm = TRUE)
+  v.r <- range(values[is.finite(values)], na.rm = TRUE)
   lags.x <- function(xs, nl){
     xs.r <- 0.5 * diff(xs/(nl-1))
     return(seq(xs[1]+xs.r, xs[2]-xs.r, l=nl))
   }
   leg.l <- list(...)
-  if(is.null(leg.l$breaks)){
-    if(is.null(leg.l$col))
-      leg.l$col <- heat.colors(12)
-    nc <- length(leg.l$col)
-    if(vertical)
-      image(x.leg, lags.x(xs=y.leg, nl=nc),
-            matrix(seq(v.r[1], v.r[2], l=nc), nrow=1),
-            add=TRUE, col=leg.l$col)    
-    else
-      image(lags.x(xs=x.leg, nl=nc), y.leg,
-            matrix(seq(v.r[1], v.r[2], l=nc), ncol=1),
-            add=TRUE, col=leg.l$col)    
-  }
-  else{
+  if(is.null(leg.l$br))
+    nc <- ifelse(is.null(leg.l$col), 12, length(leg.l$col))
+  else
     nc <- length(leg.l$breaks) - 1
-    if(is.null(leg.l$col))
-      leg.l$col <- heat.colors(nc)
-    if(vertical)
-      image(x.leg, lags.x(xs=y.leg, nl=nc),
-          matrix(seq(v.r[1], v.r[2], l=nc), nrow=1),
-          add=TRUE, col=leg.l$col, breaks=leg.l$breaks)
-    else
-      image(lags.x(xs=x.leg, nl=nc), y.leg,
-            matrix(seq(v.r[1], v.r[2], l=nc), ncol=1),
-            add=TRUE, col=leg.l$col, breaks=leg.l$breaks)
-  }
+  if(is.null(leg.l$col)) leg.l$col <- heat.colors(nc)
+  if(is.null(leg.l$zl)) leg.l$zlim <- c(v.r[1], v.r[2])
+  if(vertical)
+    xy <- list(x=x.leg, y=lags.x(xs=y.leg, nl=nc))
+  else
+    xy <- list(x=lags.x(xs=x.leg, nl=nc), y=y.leg)
+  if(is.null(leg.l$br))
+    image(x=xy$x, y=xy$y,
+          z=matrix(seq(leg.l$zlim[1], leg.l$zlim[2], l=nc), ncol=1),
+          add=TRUE, xaxs = "i", yaxs = "i", xlab="", ylab="",
+          zlim = leg.l$zlim, col=leg.l$col)
+  else
+    image(x=xy$x, y=xy$y,
+          z=matrix(seq(leg.l$zlim[1], leg.l$zlim[2], l=nc), ncol=1),
+          add=TRUE, xaxs = "i", yaxs = "i", xlab="", ylab="",
+          zlim = leg.l$zlim, col=leg.l$col, breaks = leg.l$br)
   leg.poly <- rbind(c(x.leg[1], y.leg[1]), c(x.leg[2], y.leg[1]),
                     c(x.leg[2], y.leg[2]), c(x.leg[1], y.leg[2]),
                     c(x.leg[1], y.leg[1]))
   polygon(leg.poly)
-  if(is.null(leg.l$cex))
-    leg.l$cex <- par()$cex
+  if(is.null(leg.l$cex)) leg.l$cex <- par()$cex
   if(missing(scale.vals))
-    scale.vals <- pretty(values, n=5, min.n=4)
-  scale.vals <- scale.vals[scale.vals > v.r[1] & scale.vals < v.r[2]]
+    scale.vals <- pretty(c(values,leg.l$zlim), n=5, min.n=4)
+  scale.vals <- scale.vals[scale.vals > leg.l$zlim[1] &
+                           scale.vals < leg.l$zlim[2]]
   if(vertical){
     y.r <- range(lags.x(xs=y.leg,nl=nc))
-    y.text <- y.r[1] + ((scale.vals - v.r[1]) * diff(y.r))/diff(v.r)
-    text((max(x.leg)+(diff(x.leg)/2)), y.text, lab=scale.vals, col=1, cex=leg.l$cex)
+    y.text <- y.r[1] + ((scale.vals - leg.l$zlim[1]) * diff(y.r))/diff(leg.l$zlim)
+    text((max(x.leg)+(diff(x.leg)/2)), y.text,
+         lab=scale.vals, col=1, cex=leg.l$cex)
   }
   else{
     x.r <- range(lags.x(xs=x.leg,nl=nc))
-    x.text <- x.r[1] + ((scale.vals - v.r[1]) * diff(x.r))/diff(v.r)
+    x.text <- x.r[1] + ((scale.vals - leg.l$zlim[1]) * diff(x.r))/diff(leg.l$zlim)
     text(x.text, (max(y.leg)+(diff(y.leg)/2)), lab=scale.vals, col=1, cex=leg.l$cex)
   }
   return(invisible())
