@@ -1,3 +1,12 @@
+##
+## Functions for likelihood based inference in the geoR package
+## ------------------------------------------------------------
+##
+## Includes:
+##    - the main function for ML estimation - likfit()
+##    - methods for class likGRF
+##    - some other functions
+
 "likfit" <-
   function (geodata, coords=geodata$coords, data=geodata$data,
             trend = "cte", ini.cov.pars,
@@ -102,8 +111,7 @@
   ## best initial value
   ##
   if(is.matrix(ini.cov.pars) | (length(nugget) > 1) | (length(kappa) > 1) | (length(lambda) > 1) | (length(psiR) > 1) | (length(psiA) > 1)){
-    if(messages.screen)
-      cat("likfit: searching for best initial value ...")
+    if(messages.screen) cat("likfit: searching for best initial value ...")
     ini.temp <- matrix(ini.cov.pars, ncol=2)
     grid.ini <- as.matrix(expand.grid(sigmasq=unique(ini.temp[,1]), phi=unique(ini.temp[,2]), tausq=unique(nugget), kappa=unique(kappa), lambda=unique(lambda), psiR=unique(psiR), psiA=unique(psiA)))
     .likGRF.dists.vec <<- lapply(split(as.data.frame(coords), realisations), vecdist)
@@ -120,10 +128,11 @@
                         realisations = realisations))
     grid.lik <- apply(grid.ini, 1, temp.f, coords = coords,
                       data = data, temp.list = temp.list)
-    grid.lik <- grid.lik[(grid.lik != Inf) & (grid.lik != -Inf) & !is.na(grid.lik) & !is.nan(grid.lik)] 
     grid.ini <- grid.ini[(grid.lik != Inf) & (grid.lik != -Inf) & !is.na(grid.lik) & !is.nan(grid.lik),, drop=FALSE] 
+    grid.lik <- grid.lik[(grid.lik != Inf) & (grid.lik != -Inf) & !is.na(grid.lik) & !is.nan(grid.lik)] 
     ini.temp <- grid.ini[which(grid.lik == max(grid.lik)),, drop=FALSE]
-    if(is.R()) rownames(ini.temp) <- "initial.value"
+    if(all(ini.temp[,"phi"] == 0)) ini.temp <- ini.temp[1,, drop=FALSE]
+    rownames(ini.temp) <- "initial.value"
     if(messages.screen){
       cat(" selected values:\n")
       print(rbind(round(ini.temp, dig=2), status=ifelse(c(FALSE, FALSE, fix.nugget, fix.kappa, fix.lambda, fix.psiR, fix.psiA), "fix", "est")))
@@ -1071,13 +1080,16 @@
     xivy <- crossprod(sivx, sivy)
     betahat <- solve.geoR(xivx, xivy)
     if(inherits(betahat, "try-error")){
-      ## error.now <- options()$show.error.message
-      ## options(show.error.messages = FALSE)
-      require(methods)
       t.ei <- eigen(xivx, symmetric = TRUE)
-      betahat <- trySilent(t.ei$vec %*% diag(t.ei$val^(-1)) %*% t(t.ei$vec) %*% xivy)
-      ##if(is.null(error.now) || error.now == TRUE)
-      ##  options(show.error.messages = TRUE)        
+      require(methods)
+      if(exists("trySilent"))
+        betahat <- trySilent(t.ei$vec %*% diag(t.ei$val^(-1)) %*% t(t.ei$vec) %*% xivy)
+      else{
+        error.now <- options()$show.error.message
+        options(show.error.messages = FALSE)
+        betahat <- try(t.ei$vec %*% diag(t.ei$val^(-1)) %*% t(t.ei$vec) %*% xivy)
+        if(is.null(error.now) || error.now == TRUE) options(show.error.messages = TRUE)        
+      }
     }
     if(inherits(betahat, "try-error"))
       stop("Covariates have very different orders of magnitude. Try to multiply and/or divide them to bring them to similar orders of magnitude") 
@@ -1208,7 +1220,7 @@
 "print.summary.likGRF" <-
   function(x, digits = "default", ...)
 {
-  if(class(x) != "summary.likGRF")
+  if(length(class(x)) == 0 || all(class(x) != "summary.likGRF"))
     stop("object is not of the class \"summary.likGRF\"")
   if(is.R() & digits == "default") digits <- max(3, getOption("digits") - 3)
   else digits <- options()$digits
@@ -1331,14 +1343,11 @@
   vecdist <- function(x){as.vector(dist(x))}
   if(psiR != 1 | psiA != 0){
     coords.c <- coords.aniso(coords, aniso.pars=c(psiA, psiR))
-    vecdist <- function(x){as.vector(dist(x))}
     .likGRF.dists.vec <- lapply(split(as.data.frame(coords.c),
                                       as.factor(realisations)), vecdist)
   }
-  else
-    if(compute.dists)
-      .likGRF.dists.vec <- lapply(split(as.data.frame(coords),
-                                        as.factor(realisations)), vecdist)
+  else if(compute.dists) .likGRF.dists.vec <- lapply(split(as.data.frame(coords),
+                                                           as.factor(realisations)), vecdist)
   ##
   ## Box-Cox transformation
   ##
@@ -1413,4 +1422,180 @@
 }
 
 
+##
+## ----------------
+## Other functions
+## ----------------
+##
+##
+
+"likfit.nospatial" <-
+  function(temp.list, ...)
+{
+  results <- list()
+  z <- temp.list$z
+  n <- temp.list$n
+  beta.size <- temp.list$beta.size
+  xmat <- temp.list$xmat
+  txmat <- temp.list$txmat
+  ixx <- solve(crossprod(xmat))
+  if(temp.list$fix.lambda == FALSE){
+    if (temp.list$minimisation.function == "nlm"){
+      assign(".temp.lower.lambda",-2, pos=1)
+      assign(".temp.upper.lambda", 2, pos=1)
+      results <- nlm(proflik.lambda, 1, ...)
+      if(exists(".temp.lambda")){
+        results$lambda <- .temp.lambda
+        remove(".temp.lambda", pos=1, inherits = TRUE)
+      }
+      else{
+        results$lambda <- results$estimate
+      }
+      rm(.temp.lower.lambda, .temp.upper.lambda, inherits = TRUE, pos=1)
+    }
+    if (temp.list$minimisation.function == "nlmP"){
+      results <- nlmP(proflik.lambda, 1, lower=-2, upper=2,...)  
+      results$lambda <- results$estimate
+    }
+    if (temp.list$minimisation.function == "optim"){
+      results <- optim(1, proflik.lambda, method="L-BFGS-B", lower=-2, upper=2,...)
+      results$minimum <- results$value
+      results$lambda <- results$par
+    }
+    if(results$lambda == 1) {
+      temp.list$log.jacobian <- 0
+    }
+    else {
+      if(any(z <= 0))
+        stop("Transformation option not allowed when there are zeros or negative data")
+      if(any(z^(results$lambda - 1) <= 0))
+        temp.list$log.jacobian <- log(prod(z^(results$lambda - 1)))
+      else temp.list$log.jacobian <- sum(log(z^(results$lambda - 1)))
+      if(results$lambda == 0)
+        z <- log(z)
+      else z <- ((z^results$lambda) - 1)/results$lambda
+    }
+  }
+  else{
+    results$lambda <- temp.list$lambda
+    results$code <- 1
+    if (temp.list$minimisation.function == "optim") results$convergence <- 0
+  }
+  ssres <- (z %*% (diag(n) - xmat %*%
+                   solve(crossprod(xmat)) %*% txmat) %*% z)
+  if(temp.list$method == "ML"){
+    results$tausqhat <- ssres/n
+    if(temp.list$fix.lambda)
+      results$minimum <- as.vector(((n/2) * log(2 * pi) +
+                          (n/2) * log(results$tausqhat) +
+                          (n/2)  -
+                          temp.list$log.jacobian))
+  }
+  if(temp.list$method == "RML") {
+    results$tausqhat  <- (ssres/(n-beta.size))
+    if(temp.list$fix.lambda){
+      results$minimum <- as.vector((((n - beta.size)/2) * log(2 * pi) +
+                          ((n - beta.size)/2) * log(results$tausqhat) +
+                          (n/2) -
+                          temp.list$log.jacobian
+                          ))
+    }
+  }
+  if (temp.list$minimisation.function == "optim") results$value <- results$minimum    
+  return(results)
+}
+
+"loglik.spatial" <- function(pars)
+{
+  tausq <- pars[1]
+  sigmasq <- pars[2]
+  sill.total <- tausq + sigmasq
+  phi <- pars[3]
+  lambda <- pars[4]
+  z <- .temp.list$z
+  n <- .temp.list$n
+  if(.temp.list$fix.lambda == FALSE) {
+    if(lambda == 1) {
+      .temp.list$log.jacobian <- 0
+    }
+    else {
+      if(any(z < 0))
+        stop("Transformation option not allowed when there are zeros or negative data"
+             )
+      if(any(z^(lambda - 1) <= 0))
+        .temp.list$log.jacobian <- log(prod(z^(lambda -
+                                               1)))
+      else .temp.list$log.jacobian <- sum(log(z^(lambda - 1)))
+      if(lambda == 0)
+        z <- log(z)
+      else z <- ((z^lambda) - 1)/lambda
+    }
+  }
+  beta.size <- .temp.list$beta.size
+  kappa <- .temp.list$kappa
+  covinf <- varcov.spatial(dists.lowertri = .temp.list$
+                           dists.lowertri, cov.model = .temp.list$cov.model,
+                           kappa = kappa, nugget = tausq,
+                           cov.pars = c(sigmasq, phi), scaled = FALSE,
+                           inv = TRUE, det = TRUE,
+                           only.inv.lower.diag = TRUE)
+  xix <- as.double(rep(0, beta.size*beta.size))
+  xix <- .C("bilinearform_XAY",
+            as.double(covinf$lower.inverse),
+            as.double(covinf$diag.inverse),
+            as.double(as.vector(.temp.list$xmat)),
+            as.double(as.vector(.temp.list$xmat)),
+            as.integer(beta.size),
+            as.integer(beta.size),
+            as.integer(n),
+            res = xix, PACKAGE = "geoR")$res
+  attr(xix, "dim") <- c(beta.size, beta.size)
+  if(length(as.vector(xix)) == 1) {
+    ixix <- 1/xix
+    choldet <- 0.5 * log(xix)
+  }
+  else {
+    chol.xix <- chol(xix)
+    ixix <- chol2inv(chol.xix)
+    choldet <- sum(log(diag(chol.xix)))
+  }
+  xiy <- as.double(rep(0, beta.size))
+  xiy <- .C("bilinearform_XAY",
+            as.double(covinf$lower.inverse),
+            as.double(covinf$diag.inverse),
+            as.double(as.vector(.temp.list$xmat)),
+            as.double(as.vector(z)),
+            as.integer(beta.size),
+            as.integer(1),
+            as.integer(n),
+            res = xiy, PACKAGE = "geoR")$res
+  beta.hat <- as.vector(ixix %*% xiy)
+  yiy <- as.double(0.0)
+  yiy <- .C("bilinearform_XAY",
+            as.double(covinf$lower.inverse),
+            as.double(covinf$diag.inverse),
+            as.double(as.vector(z)),
+            as.double(as.vector(z)),
+            as.integer(1),
+            as.integer(1),
+            as.integer(n),
+            res = yiy, PACKAGE = "geoR")$res
+  ssresmat <- as.vector(yiy - 2*crossprod(beta.hat,xiy) +  beta.hat %*% xix %*% beta.hat)
+  if(.temp.list$method == "ML") {
+    loglik <- ( - (n/2) * log(2 * pi) -
+               covinf$log.det.to.half -
+               0.5 * ssresmat + 
+               .temp.list$log.jacobian)
+  }
+  if(.temp.list$method == "RML") {
+    xx.eigen <- eigen(crossprod(.temp.list$xmat), symmetric = TRUE, only.values = TRUE)
+    loglik <- ( - ((n - beta.size)/2) * log(2 * pi) +
+               0.5 * sum(log(xx.eigen$values)) -
+               covinf$log.det.to.half -
+               (0.5) * ssresmat -
+               choldet +
+               .temp.list$log.jacobian)
+  }
+  return(as.vector(loglik))
+}
 
